@@ -2,6 +2,14 @@ import collections
 import json
 import random
 
+import botocore.exceptions
+import requests
+from auto_labeling_pipeline.menu import Options
+from auto_labeling_pipeline.models import RequestModelFactory
+from auto_labeling_pipeline.mappings import MappingTemplate
+from auto_labeling_pipeline.task import TaskFactory
+from auto_labeling_pipeline.postprocessing import PostProcessor
+from auto_labeling_pipeline.pipeline import pipeline
 from django.conf import settings
 from django.contrib.auth.models import User
 from django.db import transaction
@@ -19,14 +27,15 @@ from rest_framework.views import APIView
 from rest_framework.parsers import MultiPartParser
 from rest_framework_csv.renderers import CSVRenderer
 
+from .exceptions import AutoLabelingException, AutoLabeliingPermissionDenied, URLConnectionError, AWSTokenError, SampleDataException
 from .filters import DocumentFilter
-from .models import Project, Label, Document, RoleMapping, Role
-from .permissions import IsProjectAdmin, IsAnnotatorAndReadOnly, IsAnnotator, IsAnnotationApproverAndReadOnly, IsOwnAnnotation, IsAnnotationApprover
-from .serializers import ProjectSerializer, LabelSerializer, DocumentSerializer, UserSerializer, ApproverSerializer
-from .serializers import ProjectPolymorphicSerializer, RoleMappingSerializer, RoleSerializer
-from .utils import CSVParser, ExcelParser, JSONParser, PlainTextParser, CoNLLParser, AudioParser, iterable_to_io
-from .utils import JSONLRenderer
-from .utils import JSONPainter, CSVPainter
+from .models import Project, Label, Document, RoleMapping, Role, Comment, AutoLabelingConfig
+from .permissions import IsProjectAdmin, IsAnnotatorAndReadOnly, IsAnnotator, IsAnnotationApproverAndReadOnly, IsOwnAnnotation, IsAnnotationApprover, IsOwnComment
+from .serializers import ProjectSerializer, LabelSerializer, DocumentSerializer, UserSerializer, ApproverSerializer, CommentSerializer
+from .serializers import ProjectPolymorphicSerializer, RoleMappingSerializer, RoleSerializer, AutoLabelingConfigSerializer
+from .utils import CSVParser, ExcelParser, JSONParser, PlainTextParser, FastTextParser, CoNLLParser, AudioParser, iterable_to_io
+from .utils import JSONLRenderer, PlainTextRenderer
+from .utils import JSONPainter, CSVPainter, FastTextPainter
 
 IsInProjectReadOnlyOrAdmin = (IsAnnotatorAndReadOnly | IsAnnotationApproverAndReadOnly | IsProjectAdmin)
 IsInProjectOrAdmin = (IsAnnotator | IsAnnotationApprover | IsProjectAdmin)
@@ -184,6 +193,12 @@ class DocumentList(generics.ListCreateAPIView):
         project = get_object_or_404(Project, pk=self.kwargs['project_id'])
         serializer.save(project=project)
 
+    def delete(self, request, *args, **kwargs):
+        project = get_object_or_404(Project, pk=self.kwargs['project_id'])
+        queryset = project.documents
+        queryset.all().delete()
+        return Response(status=status.HTTP_204_NO_CONTENT)
+
 
 class DocumentDetail(generics.RetrieveUpdateDestroyAPIView):
     queryset = Document.objects.all()
@@ -221,6 +236,11 @@ class AnnotationList(generics.ListCreateAPIView):
     def perform_create(self, serializer):
         serializer.save(document_id=self.kwargs['doc_id'], user=self.request.user)
 
+    def delete(self, request, *args, **kwargs):
+        queryset = self.get_queryset()
+        queryset.all().delete()
+        return Response(status=status.HTTP_204_NO_CONTENT)
+
     @staticmethod
     def check_single_class_classification(project_id, doc_id, user):
         project = get_object_or_404(Project, pk=project_id)
@@ -260,6 +280,27 @@ class AnnotationDetail(generics.RetrieveUpdateDestroyAPIView):
         return self.queryset
 
 
+class CommentList(generics.ListCreateAPIView):
+    serializer_class = CommentSerializer
+    permission_classes = [IsAuthenticated & IsInProjectOrAdmin]
+
+    def get_queryset(self):
+        return Comment.objects.filter(
+            document_id=self.kwargs['doc_id'],
+            user_id=self.request.user.id,
+        ).all()
+
+    def perform_create(self, serializer):
+        serializer.save(user=self.request.user, document_id=self.kwargs['doc_id'])
+
+
+class CommentDetail(generics.RetrieveUpdateDestroyAPIView):
+    queryset = Comment.objects.all()
+    serializer_class = CommentSerializer
+    lookup_url_kwarg = 'comment_id'
+    permission_classes = [IsAuthenticated & IsInProjectOrAdmin & IsOwnComment]
+
+
 class TextUploadAPI(APIView):
     parser_classes = (MultiPartParser,)
     permission_classes = [IsAuthenticated & IsProjectAdmin]
@@ -299,6 +340,8 @@ class TextUploadAPI(APIView):
             return ExcelParser()
         elif file_format == 'audio':
             return AudioParser()
+        elif file_format == 'fastText':
+            return FastTextParser()
         else:
             raise ValidationError('format {} is invalid.'.format(file_format))
 
@@ -357,19 +400,25 @@ class CloudUploadAPI(APIView):
 class TextDownloadAPI(APIView):
     permission_classes = TextUploadAPI.permission_classes
 
-    renderer_classes = (CSVRenderer, JSONLRenderer)
+    renderer_classes = (CSVRenderer, JSONLRenderer, PlainTextRenderer)
 
     def get(self, request, *args, **kwargs):
         format = request.query_params.get('q')
+        only_approved = request.query_params.get('onlyApproved')
         project = get_object_or_404(Project, pk=self.kwargs['project_id'])
-        documents = project.documents.all()
+        documents = (
+            project.documents.exclude(annotations_approved_by = None)
+            if only_approved == 'true'
+            else project.documents.all()
+        )
         painter = self.select_painter(format)
-        # json1 format prints text labels while json format prints annotations with label ids
-        # json1 format - "labels": [[0, 15, "PERSON"], ..]
-        # json format - "annotations": [{"label": 5, "start_offset": 0, "end_offset": 2, "user": 1},..]
-        if format == "json1":
+
+        # jsonl-textlabel format prints text labels while jsonl format prints annotations with label ids
+        # jsonl-textlabel format - "labels": [[0, 15, "PERSON"], ..]
+        # jsonl format - "annotations": [{"label": 5, "start_offset": 0, "end_offset": 2, "user": 1},..]
+        if format in ('jsonl', 'txt'):
             labels = project.labels.all()
-            data = JSONPainter.paint_labels(documents, labels)
+            data = painter.paint_labels(documents, labels)
         else:
             data = painter.paint(documents)
         return Response(data)
@@ -377,8 +426,10 @@ class TextDownloadAPI(APIView):
     def select_painter(self, format):
         if format == 'csv':
             return CSVPainter()
-        elif format == 'json' or format == "json1":
+        elif format == 'jsonl' or format == 'json':
             return JSONPainter()
+        elif format == 'txt':
+            return FastTextPainter()
         else:
             raise ValidationError('format {} is invalid.'.format(format))
 
@@ -439,3 +490,227 @@ class LabelUploadAPI(APIView):
         except IntegrityError:
             content = {'error': 'IntegrityError: you cannot create a label with same name or shortkey.'}
             return Response(content, status=status.HTTP_400_BAD_REQUEST)
+
+
+class AutoLabelingTemplateListAPI(APIView):
+    permission_classes = [IsAuthenticated & IsProjectAdmin]
+
+    def get(self, request, *args, **kwargs):
+        project = get_object_or_404(Project, pk=self.kwargs['project_id'])
+        options = Options.filter_by_task(task_name=project.project_type)
+        option_names = [o.name for o in options]
+        return Response(option_names, status=status.HTTP_200_OK)
+
+
+class AutoLabelingTemplateDetailAPI(APIView):
+    permission_classes = [IsAuthenticated & IsProjectAdmin]
+
+    def get(self, request, *args, **kwargs):
+        option_name = self.kwargs['option_name']
+        option = Options.find(option_name=option_name)
+        return Response(option.to_dict(), status=status.HTTP_200_OK)
+
+
+class AutoLabelingConfigList(generics.ListCreateAPIView):
+    serializer_class = AutoLabelingConfigSerializer
+    pagination_class = None
+    permission_classes = [IsAuthenticated & IsProjectAdmin]
+
+    def get_queryset(self):
+        project = get_object_or_404(Project, pk=self.kwargs['project_id'])
+        return project.auto_labeling_config
+
+    def perform_create(self, serializer):
+        project = get_object_or_404(Project, pk=self.kwargs['project_id'])
+        serializer.save(project=project)
+
+
+class AutoLabelingConfigDetail(generics.RetrieveUpdateDestroyAPIView):
+    queryset = AutoLabelingConfig.objects.all()
+    serializer_class = AutoLabelingConfigSerializer
+    lookup_url_kwarg = 'config_id'
+    permission_classes = [IsAuthenticated & IsProjectAdmin]
+
+
+class AutoLabelingConfigTest(APIView):
+    permission_classes = [IsAuthenticated & IsProjectAdmin]
+
+    def post(self, *args, **kwargs):
+        try:
+            output = self.pass_config_validation()
+            output = self.pass_pipeline_call(output)
+            if not output:
+                raise SampleDataException()
+            return Response(
+                data={'valid': True, 'labels': output},
+                status=status.HTTP_200_OK
+            )
+        except requests.exceptions.ConnectionError:
+            raise URLConnectionError()
+        except botocore.exceptions.ClientError:
+            raise AWSTokenError()
+        except ValidationError as e:
+            raise e
+        except Exception as e:
+            raise e
+
+    def pass_config_validation(self):
+        config = self.request.data['config']
+        serializer = AutoLabelingConfigSerializer(data=config)
+        serializer.is_valid(raise_exception=True)
+        return serializer
+
+    def pass_pipeline_call(self, serializer):
+        test_input = self.request.data['input']
+        project = get_object_or_404(Project, pk=self.kwargs['project_id'])
+        return execute_pipeline(
+            text=test_input,
+            project_type=project.project_type,
+            model_name=serializer.data.get('model_name'),
+            model_attrs=serializer.data.get('model_attrs'),
+            template=serializer.data.get('template'),
+            label_mapping=serializer.data.get('label_mapping')
+        )
+
+
+class AutoLabelingConfigParameterTest(APIView):
+    permission_classes = [IsAuthenticated & IsProjectAdmin]
+
+    def post(self, *args, **kwargs):
+        model_name = self.request.data['model_name']
+        model_attrs = self.request.data['model_attrs']
+        sample_text = self.request.data['text']
+        try:
+            model = RequestModelFactory.create(model_name, model_attrs)
+        except Exception:
+            model = RequestModelFactory.find(model_name)
+            schema = model.schema()
+            required_fields = ', '.join(schema['required']) if 'required' in schema else ''
+            raise ValidationError(
+                'The attributes does not match the model.'
+                'You need to correctly specify the required fields: {}'.format(required_fields)
+            )
+        try:
+            request = model.build()
+            response = request.send(text=sample_text)
+            return Response(response, status=status.HTTP_200_OK)
+        except requests.exceptions.ConnectionError:
+            raise URLConnectionError
+        except botocore.exceptions.ClientError:
+            raise AWSTokenError()
+        except Exception as e:
+            raise e
+
+
+class AutoLabelingTemplateTest(APIView):
+    permission_classes = [IsAuthenticated & IsProjectAdmin]
+
+    def post(self, *args, **kwargs):
+        response = self.request.data['response']
+        template = self.request.data['template']
+        project = get_object_or_404(Project, pk=self.kwargs['project_id'])
+        task = TaskFactory.create(project.project_type)
+        template = MappingTemplate(
+            label_collection=task.label_collection,
+            template=template
+        )
+        labels = template.render(response)
+        if not labels.dict():
+            raise SampleDataException()
+        return Response(labels.dict(), status=status.HTTP_200_OK)
+
+
+class AutoLabelingMappingTest(APIView):
+    permission_classes = [IsAuthenticated & IsProjectAdmin]
+
+    def post(self, *args, **kwargs):
+        response = self.request.data['response']
+        label_mapping = self.request.data['label_mapping']
+        project = get_object_or_404(Project, pk=self.kwargs['project_id'])
+        task = TaskFactory.create(project.project_type)
+        labels = task.label_collection(response)
+        post_processor = PostProcessor(label_mapping)
+        labels = post_processor.transform(labels)
+        return Response(labels.dict(), status=status.HTTP_200_OK)
+
+
+class AutoLabelingAnnotation(generics.CreateAPIView):
+    pagination_class = None
+    permission_classes = [IsAuthenticated & IsInProjectOrAdmin]
+    swagger_schema = None
+
+    def get_serializer_class(self):
+        project = get_object_or_404(Project, pk=self.kwargs['project_id'])
+        self.serializer_class = project.get_annotation_serializer()
+        return self.serializer_class
+
+    def get_queryset(self):
+        project = get_object_or_404(Project, pk=self.kwargs['project_id'])
+        model = project.get_annotation_class()
+        queryset = model.objects.filter(document=self.kwargs['doc_id'])
+        if not project.collaborative_annotation:
+            queryset = queryset.filter(user=self.request.user)
+        return queryset
+
+    def create(self, request, *args, **kwargs):
+        queryset = self.get_queryset()
+        if queryset.exists():
+            raise AutoLabelingException()
+        labels = self.extract()
+        labels = self.transform(labels)
+        serializer = self.get_serializer(data=labels, many=True)
+        serializer.is_valid(raise_exception=True)
+        self.perform_create(serializer)
+        headers = self.get_success_headers(serializer.data)
+        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)
+
+    def perform_create(self, serializer):
+        serializer.save(user=self.request.user)
+
+    def extract(self):
+        project = get_object_or_404(Project, pk=self.kwargs['project_id'])
+        doc = get_object_or_404(Document, pk=self.kwargs['doc_id'])
+        config = project.auto_labeling_config.first()
+        if not config:
+            raise AutoLabeliingPermissionDenied()
+        return execute_pipeline(
+            text=doc.text,
+            project_type=project.project_type,
+            model_name=config.model_name,
+            model_attrs=config.model_attrs,
+            template=config.template,
+            label_mapping=config.label_mapping
+        )
+
+    def transform(self, labels):
+        project = get_object_or_404(Project, pk=self.kwargs['project_id'])
+        for label in labels:
+            label['document'] = self.kwargs['doc_id']
+            if 'label' in label:
+                label['label'] = project.labels.get(text=label.pop('label')).id
+        return labels
+
+
+def execute_pipeline(text: str,
+                     project_type: str,
+                     model_name: str,
+                     model_attrs: dict,
+                     template: str,
+                     label_mapping: dict):
+    task = TaskFactory.create(project_type)
+    model = RequestModelFactory.create(
+        model_name=model_name,
+        attributes=model_attrs
+    )
+    template = MappingTemplate(
+        label_collection=task.label_collection,
+        template=template
+    )
+    post_processor = PostProcessor(label_mapping)
+    labels = pipeline(
+        text=text,
+        request_model=model,
+        mapping_template=template,
+        post_processing=post_processor
+    )
+    return labels.dict()
